//  2d Feeding Optimization
//  Copyright 8/18/00 Andrew Lovett, under the supervision of Louis Gross
//  Check the file Info/CopyrightInfo.txt for copyright information

//This is a somewhat more complex animal than the horse or koala; it can see beyond //the adjacent spaces, and it compares its own feeding space to those other spaces

import swarm.Globals;
import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.defobj.Zone;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;
import swarm.Selector;

public class Horse extends Animal {

	Schedule schedule; //we use the schedule object to schedule the actions

	//the three following variables are all found in Animal, Cow's parent class, 	//but they have been added to this class so that they will show up whenever the 	//user probes an animal of this type
	int age;
	int energy;
	boolean alive;

	//Horse Stats, see the Animal class for info on what each stat means
	//If you wish to modify these stats, go to Data\HorseData.txt
	//As these values will be modified only once, at the beginning of a simulation, 	//we will keep them in all-caps, as though they were constants
	public static int SPEED;
	public static int SIGHT;
	public static int SIZE;
	public static int MAXEN;
	public static int BABYAGE;
	public static int OLDAGE;
	public static int NUMBABIES;
	public static int ENTOMOVE;
	public static int ENFROMFOOD;
	public static int ENFORBABY;
	public static byte COLOR; 

	//the constructor begins by sending info on all these stats to the parent 		//class, Animal
	public Horse (Zone thZone, Landscape thWorld, int x, int y) {
		super (thZone, thWorld, SPEED, SIGHT, SIZE, x, y, COLOR, MAXEN, BABYAGE, 					OLDAGE, NUMBABIES, ENTOMOVE, ENFROMFOOD, ENFORBABY);

		buildActions (thZone); //set up our schedule
	}

	//here's our schedule; the frequency of the schedule's repeats depends upon the 	//speed of the animal
	public Object buildActions (Zone thZone) {
		schedule = new ScheduleImpl (thZone, speed);
		try {
			schedule.at$createActionTo$message (0, this, 
						new Selector (getClass (), "takeStep", false));
		}  
		catch (Exception e) {
			e.printStackTrace (System.err);
			System.exit (1);
		}

		return this;
	}

	//each time it takes a turn, assuming it is still alive, the animal updates 	//age, energy, and alive; moves to the best food space (either the one with the 	//most food, or one one the way to the one with the most food); eats the food; 	//and grows
	public void takeStep () {
		if (super.alive) {
			this.age = super.age;
			this.energy = super.energy;
			this.alive = true;

			moveToBestFood ();
			eatFood ();
			grow ();
		} else
			this.alive = false;
	}

	//here we activate the animal and its schedule
	public Activity activateIn (Swarm papaswarm) {
		super.activateIn (papaswarm);
		schedule.activateIn (this);
		return getActivity ();
	}
}
